#!/bin/sh

RF_MACPORTS_VERSION=2.3.1
RF_DARWIN_VERSION=`uname -r`
RF_MACPORTS_PREFIX=${1}
RF_MACPORTS_CONFIGURE="./configure --prefix=${RF_MACPORTS_PREFIX}"

if [ -z "${RF_MACPORTS_PREFIX}" ] ; then
echo `dirname ${0}`/setup-mac.sh or `dirname ${0}`/setup-mac-llvm.sh
exit 1
fi

for CC in clang llvm-gcc-4.2 gcc-4.2 ; do
if ${CC} --version  > /dev/null 2>&1 ; then
RF_DEFAULT_COMPILER=${CC}
fi
done

if [ -z "${RF_DEFAULT_COMPILER}" ] ; then
echo compiler not found. please install Command Line Tools \(or Xcode\).
exit 1
fi
if [ ${RF_DEFAULT_COMPILER} != gcc-4.2 ] ; then
RF_FORCE_COMPILER=
fi
if [ "${RF_FORCE_COMPILER}" ] ; then
RF_MACPORTS_CONFIGURE="${RF_MACPORTS_CONFIGURE} CC=${RF_DEFAULT_COMPILER}"
fi

JAVA_HOME=`/usr/libexec/java_home -v 1.6 2>/dev/null`
if [ "${JAVA_HOME}" ] ; then
export PATH="${JAVA_HOME}/bin:${PATH}"
fi
if ! javac -version > /dev/null 2>&1 ; then
echo javac not found. please install Java for OS X.
exit 1
fi

RF_XCODE_VERSION=`xcodebuild -version | grep ^Xcode | awk '{print $2}'`
if [ -z "${RF_XCODE_VERSION}" ] ; then
echo xcode not found. please install Xcode.
exit 1
fi

case ${RF_XCODE_VERSION} in
3.*)
RF_UNIVERSAL_ARCHS="x86_64 i386 ppc"
;;
esac

case ${RF_DARWIN_VERSION} in
13.*)
AFTER_MARVERICKS=1
;;
esac

if [ "${AFTER_MARVERICKS}" ] ; then
if ! pkgutil --pkg-info com.apple.pkg.CLTools_Executables > /dev/null 2>&1 ; then
echo compiler not found. please install Command Line Tools.
exit 1
fi
fi

cd `dirname ${0}`/../..
if [ ! -f MacPorts-${RF_MACPORTS_VERSION}.tar.bz2 ] ; then
curl -O https://distfiles.macports.org/MacPorts/MacPorts-${RF_MACPORTS_VERSION}.tar.bz2
fi
if [ -e MacPorts-${RF_MACPORTS_VERSION} ] ; then
rm -rf MacPorts-${RF_MACPORTS_VERSION}
fi
tar jxf MacPorts-${RF_MACPORTS_VERSION}.tar.bz2
cd MacPorts-${RF_MACPORTS_VERSION}

if [ "${RF_UNIVERSAL_ARCHS}" ] ; then
${RF_MACPORTS_CONFIGURE} --with-universal-archs="${RF_UNIVERSAL_ARCHS}"
else
${RF_MACPORTS_CONFIGURE}
fi
make

if [ $? != 0 ] ; then
exit 1
fi

sudo sh << EOC
make install
if [ "${RF_FORCE_COMPILER}" ] ; then
echo "default_compilers ${RF_DEFAULT_COMPILER}" >> ${RF_MACPORTS_PREFIX}/etc/macports/macports.conf
fi
if [ "${RF_DEFAULT_VARIANTS}" ] ; then
echo "${RF_DEFAULT_VARIANTS}" >> ${RF_MACPORTS_PREFIX}/etc/macports/variants.conf
fi
EOC
